
clear all
close all

% Initial (Matlab related) stuff

startup;

%
% DEFINE PARAMETERS
%

VERBOSE_LEVEL      = 1;  % the higher the more is printed at stdout
PLOTTING_LEVEL     = 1;  % the higher the more plots will appear
DO_SECURITY_BACKUP = 1;  % save matlab workspace between simulations

% decide wether to use a fake liquid (i.e. set of delay lines) or a real one

NETWORK_TYPE = 'I&F';

% input distribution parameters

TMAX           = 0.5;
NINPUTCHANNELS = 1;
JITTER         = 4e-3;
NTEMPLATES     = [2 2];
NEXAMPLES      = [800 80]; % Minimal train, test examples

% network parameters

NOISE          = [0.0e-09   0.0e-09];  % [A], interval of std of gaussian white noise
NETWORK_SIZE   = [2 2 3;
                  3 3 6;
                  5 5 4;
                  5 5 8;
                  7 7 8;
                  7 7 12;
                  7 7 16]';

% simulation parameters

RUNS          =  10;

%
% plot input disribution
%

if PLOTTING_LEVEL > 0

  % generate on example of the input distribution

  InputDist = jittered_templates('nChannels',NINPUTCHANNELS,'nTemplates',NTEMPLATES,'Tstim',TMAX,'jitter',JITTER);
  S=generate(InputDist);
  figure(1); clf reset;
  plot(InputDist,S);
  clear InputDist

  % and wait for any key
  % anykey;
end

%
% do the simulation
%

nNetworkSizes = size(NETWORK_SIZE,2);

for nRun = 1:RUNS

   %
   % define the input distribution
   %

   InputDist{nRun} = jittered_templates('nChannels',NINPUTCHANNELS,'nTemplates',NTEMPLATES,'Tstim',TMAX,'jitter',JITTER);

   for nNS = 1:nNetworkSizes

      fprintf('\nRun: %i/%i  Size: %i/%i\n',nRun,RUNS,nNS,nNetworkSizes);

      %
      % simulate circuits
      %

      % analyse only circuits that respond

      clear train_states
      train_states.X = 0;

      while ~diff(minmax([train_states.X]))

         %
         % define the neural circuit
         %

         switch(NETWORK_TYPE)
            case 'fake'
               nmc = delay_lines('nMulti',prod(NETWORK_SIZE),'delayRange',[0 20e-3],'nInputs',NINPUTCHANNELS);
            otherwise
               nmc = make_network(NETWORK_SIZE(:,nNS)',NINPUTCHANNELS,NOISE);
         end

         %
         % collect stimulus/response pairs for training
         %

         [train_response,train_stimuli] = collect_sr_data(nmc,InputDist{nRun},NEXAMPLES(1));
         [test_response,test_stimuli] = collect_sr_data(nmc,InputDist{nRun},NEXAMPLES(2));

         %
         % all readouts are trained/tested on the same set of states (for panels a,b,c)
         %

         train_states  = response2states(train_response,[],[TMAX]);
         test_states = response2states(test_response,[],[TMAX]);
      end

      %
      % The readout 'Fisher discriminant 'is also trained on random train/test states (for panel d)
      %

      rnd_train_states = train_states;
      rnd_test_states = test_states;

      for nE = 1:NEXAMPLES(1)
         rnd_train_states(nE).X =  rand(size(rnd_train_states(nE).X));
      end
      for nE = 1:NEXAMPLES(2)
         rnd_test_states(nE).X =  rand(size(rnd_test_states(nE).X));
      end

      %
      % train readout with different learning algorithms on the network responses
      %

      clear readout

      readout{1} = external_readout('noise',0,'doNorm','0',...
         'description','Delta rule',...
         'targetFunction',segment_classification,...
         'algorithm',pdelta('n',1,'rho',0.4,'maxepoch',200,'gamma',0.0,'mu',0.0,...
	                    'max_err_inc',1.01,'lr_dec',0.8,'lr_inc',1.18));

      readout{2} = external_readout('noise',0,'doNorm','0',...
          'description','Fisher discriminant',...
          'targetFunction',segment_classification,...
          'algorithm',fisher_discriminant);

      readout{3} = external_readout('noise',0,'doNorm','0',...
         'description','p-Delta rule',...
         'targetFunction',segment_classification,...
         'algorithm',pdelta('n',10,'rho',4,'maxepoch',200,'gamma',0.0,'mu',0.0,...
	                    'max_err_inc',1.01,'lr_dec',0.8,'lr_inc',1.18));


      readout{4} = external_readout('noise',0,'doNorm','0',...
          'description','Voted perceptron',...
          'targetFunction',segment_classification,...
          'algorithm',voted_perceptron('maxepoch',200,'learnrate',1,'votes',10));


      readout{5} = external_readout('noise',0,'doNorm','0',...
         'description','Backpropagation',...
         'targetFunction',segment_classification,...
         'algorithm',backprop('nHidden',10,'lr',0.1,'maxEpochs',200,...
	                      'valFrac',0.0,'trainFun','trainlm'));


      close all
      [trained_readouts, perf_train{nRun,nNS}, perf_test{nRun,nNS}] = train_readouts(readout,train_states,train_stimuli,test_states,test_stimuli);

      %
      % recalculate classification error for multi unit readouts
      % ('perf_train' above contains mse and mae of analog output and not the classification error)
      %

      perf_train{nRun,nNS}(3) = analyse_classification(trained_readouts{3},train_states,train_stimuli);
      perf_test{nRun,nNS}(3)  = analyse_classification(trained_readouts{3},test_states,test_stimuli);
      perf_train{nRun,nNS}(5) = analyse_classification(trained_readouts{5},train_states,train_stimuli);
      perf_test{nRun,nNS}(5)  = analyse_classification(trained_readouts{5},test_states,test_stimuli);


      %
      % train readout with 'Fisher discriminant' on random network states
      %

      clear readout2

      readout2 = external_readout('noise',0,'doNorm','0',...
          'description','Fisher discriminant',...
          'targetFunction',segment_classification,...
          'algorithm',fisher_discriminant);

      [trained_readout2, rnd_perf_train{nRun,nNS}, rnd_perf_test{nRun,nNS}] = train_readouts(readout2,rnd_train_states,train_stimuli,rnd_test_states,test_stimuli);

      if DO_SECURITY_BACKUP
         save('Figure_1n2.mat')
      end

   end
end



%
% plot figures
%


figure; clf
nNeurons = prod(NETWORK_SIZE);
axis_range = [0 800 0 70];

% reshape performance data arrays

ptr = vertcat(perf_train{:});
ptr = 100*reshape([ptr.mae],[size(perf_train) length(readout)]);

pte = vertcat(perf_test{:});
pte = 100*reshape([pte.mae],[size(perf_test) length(readout)]);

rnd_ptr = vertcat(rnd_perf_train{:});
rnd_ptr = 100*reshape([rnd_ptr.mae],size(rnd_perf_train));

rnd_pte = vertcat(rnd_perf_test{:});
rnd_pte = 100*reshape([rnd_pte.mae],size(rnd_perf_test));


%
% Panel a
%


subplot(2,2,1);
hold on

nReadout = 1;

mean_train = mean(ptr(:,:,nReadout),1);
sem_train = std(ptr(:,:,nReadout),0,1)/sqrt(RUNS);
mean_test = mean(pte(:,:,nReadout),1);
sem_test = std(pte(:,:,nReadout),0,1)/sqrt(RUNS);

ha(:,1) = errorbar(nNeurons,mean_train,sem_train,'b--');
ha(:,2) = errorbar(nNeurons,mean_test,sem_test,'r-');

legend(ha(2,:),'Training data','Test data')
title(get(readout{nReadout},'description'))
xlabel(sprintf('Size of the recurrent circuit (neuron #)\na'))
ylabel('Error [%]')
box on
axis(axis_range)


%
% Panel b
%


subplot(2,2,2);
hold on

nReadout = 2;

mean_train = mean(ptr(:,:,nReadout),1);
sem_train = std(ptr(:,:,nReadout),0,1)/sqrt(RUNS);
mean_test = mean(pte(:,:,nReadout),1);
sem_test = std(pte(:,:,nReadout),0,1)/sqrt(RUNS);

hb(:,1) = errorbar(nNeurons,mean_train,sem_train,'b--');
hb(:,2) = errorbar(nNeurons,mean_test,sem_test,'r-');

legend(hb(2,:),'Training data','Test data')
title(get(readout{nReadout},'description'))
xlabel(sprintf('Size of the recurrent circuit (neuron #)\nb'))
ylabel('Error [%]')
box on
axis(axis_range)


%
% Panel c
%


subplot(2,2,3);
hold on

line_prop = {'','','r-*','r:s','r-.d'};

for nReadout = 3:5
   mean_test = mean(pte(:,:,nReadout),1);
   sem_test = std(pte(:,:,nReadout),0,1)/sqrt(RUNS);
   hc(:,nReadout) = errorbar(nNeurons,mean_test,sem_test,line_prop{nReadout});
   lstr{nReadout} = get(readout{nReadout},'description');
end

legend(hc(2,3:5),lstr{3:5})
title('Multi-unit readouts')
xlabel(sprintf('Size of the recurrent circuit (neuron #)\nc'))
ylabel('Error [%]')
box on
axis(axis_range)


%
% Panel d
%


subplot(2,2,4); 
hold on

mean_train = mean(rnd_ptr(:,:),1);
sem_train = std(rnd_ptr(:,:),0,1)/sqrt(RUNS);
mean_test = mean(rnd_pte(:,:),1);
sem_test = std(rnd_pte(:,:),0,1)/sqrt(RUNS);

hd(:,1) = errorbar(nNeurons,mean_train,sem_train,'b--');
hd(:,2) = errorbar(nNeurons,mean_test,sem_test,'r-');

legend(hd(2,:),'Uniformly random training data','Uniformly random test data')
title(get(readout2,'description'))
xlabel(sprintf('Size of the recurrent circuit (neuron #)\nd'))
ylabel('Error [%]')
box on
axis(axis_range)


