
clear all
close all

% Initial (Matlab related) stuff

startup;

%
% DEFINE PARAMETERS
%

VERBOSE_LEVEL      = 2;  % the higher the more is printed at stdout
PLOTTING_LEVEL     = 1;  % the higher the more plots will appear
DO_SECURITY_BACKUP = 1;  % save matlab workspace between simulations

% decide wether to use a fake liquid (i.e. set of delay lines) or a real one

NETWORK_TYPE = 'I&F';

% input distribution parameters

TMAX           = 0.5;
NINPUTCHANNELS = 1;
JITTER         = 4e-3;
NTEMPLATES     = [2 2];
NEXAMPLES      = [800 80]; % Minimal train, test examples

% network parameters

NETWORK_SIZE   = [2 2 3;
                  3 3 6;
                  5 5 4;
                  5 5 8;
                  7 7 8;
                  7 7 12;
                  7 7 16]';

% simulation parameters

RUNS          =  20;

NL1 = 1:ceil(RUNS/2);		% runs with noise level 1
NL2 = (ceil(RUNS/2) + 1):RUNS;  % runs with noise level 2

NOISE(NL1,:) = repmat([0.5e-09   0.5e-09],[length(NL1) 1]);  % [A], interval of std of gaussian white noise
NOISE(NL2,:) = repmat([1.0e-09   1.0e-09],[length(NL2) 1]);  % [A], interval of std of gaussian white noise


%
% plot input disribution
%

if PLOTTING_LEVEL > 0

  % generate on example of the input distribution

  InputDist = jittered_templates('nChannels',NINPUTCHANNELS,'nTemplates',NTEMPLATES,'Tstim',TMAX,'jitter',JITTER);
  S=generate(InputDist);
  figure(1); clf reset;
  plot(InputDist,S);
  clear InputDist

  % and wait for any key
  % anykey;
end

%
% do the simulation
%

nNetworkSizes = size(NETWORK_SIZE,2);

for nRun = 1:RUNS

   %
   % define the input distribution
   %

   InputDist{nRun} = jittered_templates('nChannels',NINPUTCHANNELS,'nTemplates',NTEMPLATES,'Tstim',TMAX,'jitter',JITTER);

   for nNS = 1:nNetworkSizes

      fprintf('\nRun: %i/%i  Size: %i/%i\n',nRun,RUNS,nNS,nNetworkSizes);

      %
      % simulate circuits
      %

      % analyse only circuits that respond

      clear train_states
      train_states.X = 0;

      while ~diff(minmax([train_states.X]))

         %
         % define the neural circuit
         %

         switch(NETWORK_TYPE)
            case 'fake'
               nmc = delay_lines('nMulti',prod(NETWORK_SIZE),'delayRange',[0 20e-3],'nInputs',NINPUTCHANNELS);
            otherwise
               nmc = make_network(NETWORK_SIZE(:,nNS)',NINPUTCHANNELS,NOISE(nRun,:));
         end

         %
         % collect stimulus/response pairs for training
         %

         [train_response,train_stimuli] = collect_sr_data(nmc,InputDist{nRun},NEXAMPLES(1));
         [test_response,test_stimuli] = collect_sr_data(nmc,InputDist{nRun},NEXAMPLES(2));

         %
         % all readouts are trained/tested on the same set of states (for panels a,b,c)
         %

         train_states  = response2states(train_response,[],[TMAX]);
         test_states = response2states(test_response,[],[TMAX]);
      end


      %
      % train readout with different learning algorithms on the network responses
      %

      clear readout

      readout = external_readout('noise',0,'doNorm','0',...
         'description','Delta rule',...
         'targetFunction',segment_classification,...
         'algorithm',pdelta('n',1,'rho',0.4,'maxepoch',200,'gamma',0.0,'mu',0.0,...
	                    'max_err_inc',1.01,'lr_dec',0.8,'lr_inc',1.18));

      close all
      [trained_readouts, perf_train{nRun,nNS}, perf_test{nRun,nNS}] = train_readouts(readout,train_states,train_stimuli,test_states,test_stimuli);


      if DO_SECURITY_BACKUP
          save('Figure_3.mat')
      end

   end
end



%
% plot figures
%


figure; clf
nNeurons = prod(NETWORK_SIZE);
axis_range = [0 800 0 70];

% reshape performance data arrays

ptr = vertcat(perf_train{:});
ptr = 100*reshape([ptr.mae],size(perf_train));

pte = vertcat(perf_test{:});
pte = 100*reshape([pte.mae],size(perf_test));


%
% Panel a
%

subplot(2,2,1);
hold on

mean_train = mean(ptr(NL1,:),1);
sem_train = std(ptr(NL1,:),0,1)/sqrt(length(NL1));
mean_test = mean(pte(NL1,:),1);
sem_test = std(pte(NL1,:),0,1)/sqrt(length(NL1));

ha(:,1) = errorbar(nNeurons,mean_train,sem_train,'b--');
ha(:,2) = errorbar(nNeurons,mean_test,sem_test,'r-');

legend(ha(2,:),'Training data','Test data')
title(sprintf('%s; std if I_{noise} = %g nA',get(readout,'description'),NOISE(1,1)*1e9))
xlabel(sprintf('Size of the recurrent circuit (neuron #)\na'))
ylabel('Error [%]')
box on
axis(axis_range)


%
% Panel b
%

subplot(2,2,2);
hold on

mean_train = mean(ptr(NL2,:),1);
sem_train = std(ptr(NL2,:),0,1)/sqrt(length(NL2));
mean_test = mean(pte(NL2,:),1);
sem_test = std(pte(NL2,:),0,1)/sqrt(length(NL2));

ha(:,1) = errorbar(nNeurons,mean_train,sem_train,'b--');
ha(:,2) = errorbar(nNeurons,mean_test,sem_test,'r-');

legend(ha(2,:),'Training data','Test data')
title(sprintf('%s; std if I_{noise} = %g nA',get(readout,'description'),NOISE(end,1)*1e9))
xlabel(sprintf('Size of the recurrent circuit (neuron #)\nb'))
ylabel('Error [%]')
box on
axis(axis_range)


