
clear all
close all

% Initial (Matlab related) stuff

startup;

%
% DEFINE PARAMETERS
%

VERBOSE_LEVEL      = 2;  % the higher the more is printed at stdout
PLOTTING_LEVEL     = 1;  % the higher the more plots will appear
DO_SECURITY_BACKUP = 1;  % save matlab workspace between simulations

% decide wether to use a fake liquid (i.e. set of delay lines) or a real one

NETWORK_TYPE = 'I&F';

% input distribution parameters

TMAX           = 1.0;
NINPUTCHANNELS = 8;
JITTER         = 4e-3;
NTEMPLATES     = [3];
NEXAMPLES      = [3 0]; % Minimal train, test examples


% network parameters


NETWORK_SIZE   = [3 3  9;...
                  3 3 11;...
                  3 3 13;...
                  3 3 15;...
                  3 3 18;...
                  3 3 21]';

NOISE = [0.0e-09   0.0e-09];  % [A], interval of std of gaussian white noise


%
% define the input distribution
%

InputDist = jittered_templates('nChannels',NINPUTCHANNELS,'nTemplates',NTEMPLATES,'templ_selection','sequential','Tstim',TMAX,'jitter',JITTER);

%
% plot input disribution
%

if PLOTTING_LEVEL > 0

  % generate on example of the input distribution

  S=generate(InputDist);
  figure(1); clf reset;
  plot(InputDist,S);

  % and wait for any key
  %anykey;
end


%
% do the simulation
%


nNetworkSizes = size(NETWORK_SIZE,2);

for nNS = 1:nNetworkSizes

   %
   % define the neural circuit
   %

   switch(NETWORK_TYPE)
      case 'fake'
         nmc = delay_lines('nMulti',prod(NETWORK_SIZE),'delayRange',[0 20e-3],'nInputs',NINPUTCHANNELS);
      otherwise
         nmc = make_network(NETWORK_SIZE(:,nNS)',NINPUTCHANNELS,NOISE);
   end

   %
   % collect stimulus/response pairs for training
   %

   AT = [0.001:20e-3:TMAX TMAX];
   [train_response{nNS},train_stimuli] = collect_sr_data(nmc,InputDist,NEXAMPLES(1));
   train_states  = response2states(train_response{nNS},[],AT);

   %
   % Calculate weight vectors optimized for smoothness
   %

   % reshape data and its derivative and generate target vectors


   X =  vertcat(train_states(1).X(1:end-2,:),...
                train_states(2).X(1:end-2,:),...
                train_states(3).X(1:end-2,:));

   X2 = vertcat(diff(train_states(1).X,2,1),...
                diff(train_states(2).X,2,1),...
                diff(train_states(3).X,2,1))/4;

   Y = ones(size(AT,2)-2,1)*[-1 0 1];
   Y = Y(:);

   % train Fisher discriminant on smooth curves

   b = 100;
   train_set.X = b*X2+(100-b)*X;
   train_set.Y = Y;

   fd = fisher_discriminant;
   fd = train(fd,train_set);

   model = get(fd,'model');
   W = model.W;

   %
   % plot figures
   %

   figure(2);
   subplot(2,3,nNS)
   cla
   hold on

   ncl = 1;

   y = train_states(ncl).X(1:end-2,:)*W;
   plot(y(:,1),y(:,2),'b-')

   box on
   set(gca,'XTick',[])
   set(gca,'YTick',[])
   xlabel(sprintf('%i neurons\n%s',prod(NETWORK_SIZE(:,nNS)),char(96+nNS)))
end

%
% plot the circuit response
%

figure(3)
clf
for nNS = 1:nNetworkSizes;
   subplot(2,3,nNS)
   cla
   hold on
   box on

   nNeurons = prod(NETWORK_SIZE(:,nNS));
   nIdx = ceil(rand(1,10)*nNeurons);

   for i = 1:length(nIdx)
      t = train_response{nNS}(1).channel(nIdx(i)).data;
      plot(t,ones(size(t))*i,'k.')
   end

   set(gca,'YTick',[1 10])

   xlabel(sprintf('time [s]\n%i neurons',prod(NETWORK_SIZE(:,nNS))))
   ylabel('Neuron')
end

